clear;
close all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Exercise 4:

f = double(imread('coins.png'));

% compute Gaussian kernel
w = gaussian(2.5);

% blur image with kernel (in pixel) using loops
ga = blur(f, w);

% show blurred image
figure;
subplot(121)
imshow(uint8(ga))

% blur image without loops
gb = conv2(f, w, 'same');

% show blurred image (notice the difference at the border)
subplot(122)
imshow(uint8(gb))

% compute gradient using convolution
figure

subplot(121);
imshow(dx(f),[]);

subplot(122);
imshow(dy(f),[]);

function w = gaussian(sigma)

    % radius of the truncated kernel
    r = ceil(3*sigma);

    % meshgrid coordinates such that 0 is in the matrix center
    x = -r:1:r;
    [X,Y] = meshgrid(x);
    
    % compute kernel elements
    w = exp( -(X.^2+Y.^2)/(2*sigma^2) );
    
    % normalize the kernel to sum up to 1
    w = w / sum(w(:));    
end

function g = blur(f, w)

    % show kernel
    %figure;
    %imshow(w/max(max(w)));
    
    r = (size(w,1)-1)/2;

    [height, width] = size(f);
    g = zeros(height, width);

    % compute convolution
    for y = 1:height
        for x = 1:width
            for i = 1:2*r
                for j = 1:2*r
                    m = i - r - 1;
                    n = j - r - 1;
                    uy = y - m;
                    ux = x - n;
                    % Truncate indices outside the image. This corresponds 
                    % to padding with the value at the image edge.
                    uy = min(height,(max(1,uy)));
                    ux = min(width, (max(1,ux)));
                    g(y,x) = g(y,x) + w(i,j) * f(uy,ux);
                end
            end
        end
    end
end

function d = dx(f)
    d = conv2(f, [1, -1], 'same');
    d(:,end) = 0;
end

function d = dy(f)
    d = conv2(f, [1; -1], 'same');
    d(end,:) = 0;
end
